@@ -2,6 +2,7 @@
 from __future__ import absolute_import, division, print_function, unicode_literals
 
 import re
+from calendar import isleap, monthrange
 from functools import wraps
 
 from six import integer_types, string_types
@@ -9,9 +10,10 @@ from six.moves import range, zip_longest
 
 from efc.rpn_builder.parser.operands import (
     BadReference, CellRangeOperand, CellSetOperand, EmptyOperand, ErrorOperand, NamedRangeOperand, NotFoundErrorOperand,
-    RPNOperand, SetOperand, SimpleOperand, SimpleSetOperand, SingleCellOperand, ValueErrorOperand, ValueNotAvailable,
+    NumErrorOperand, RPNOperand, SetOperand, SimpleOperand, SimpleSetOperand, SingleCellOperand, ValueErrorOperand,
+    ValueNotAvailable,
 )
-from efc.utils import is_float
+from efc.utils import is_float, parse_date
 
 __all__ = ('EXCEL_FUNCTIONS',)
 
@@ -409,7 +411,7 @@ def ifs_indexes(*args):
         check_good_indexes = None
         key = None
         if op_range.source and isinstance(op_range, CellRangeOperand):
-            cache = op_range.source.ifs_range_cache
+            cache = op_range.source._caches['ifs'] if op_range.source._caches is not None else None
 
             if cache is not None:
                 key = (op_range.ws_name, op_range.row1, op_range.column1, op_range.row2,
@@ -600,6 +602,73 @@ def search_func(pattern, source, start_position=None):
         return ValueErrorOperand()
 
 
+TRIM_REGEXP = re.compile(r' {2,}')
+
+
+def trim_func(op):
+    value = op.string.strip()
+    value = TRIM_REGEXP.sub(' ', value)
+    return value
+
+
+def len_func(op):
+    value = op.string
+    return len(value)
+
+
+def year_frac(dt1, dt2, tp=None):
+    if isinstance(tp, EmptyOperand) or tp is None:
+        tp = 0
+    else:
+        tp = tp.digit
+
+    if tp in (0, 4):
+        dt1p = parse_date(dt1.string)
+        dt2p = parse_date(dt2.string)
+
+        dt1p_day = dt1p.day
+        dt2p_day = dt2p.day
+
+        # https://en.wikipedia.org/wiki/Day_count_convention#30/360_US
+        if tp == 0:
+            lfd1 = monthrange(dt1p.year, dt1p.month)[1]
+            lfd2 = monthrange(dt2p.year, dt1p.month)[1]
+            if dt1p_day == lfd1 and dt2p_day == lfd2:
+                dt2p_day = 30
+
+            if dt1p_day == lfd1:
+                dt1p_day = 30
+
+            if dt2p_day == 31 and dt1p_day >= 30:
+                dt2p_day = 30
+
+            if dt1p_day == 31:
+                dt1p_day = 30
+
+        # https://en.wikipedia.org/wiki/Day_count_convention#30E/360
+        else:
+            if dt1p_day == 31:
+                dt1p_day = 30
+
+            if dt2p_day == 31:
+                dt2p_day = 30
+
+        # https://en.wikipedia.org/wiki/Day_count_convention#30/360_methods
+        v = (360 * (dt2p.year - dt1p.year) + 30 * (dt2p.month - dt1p.month) + (dt2p_day - dt1p_day)) / 360
+    elif tp in (1, 2, 3):
+        if tp == 1:
+            year_days = 366 if isleap(parse_date(dt2.string).year) else 365
+        elif tp == 2:
+            year_days = 360
+        else:
+            year_days = 365
+        v = (int(dt2.digit) - int(dt1.digit)) / year_days
+    else:
+        v = NumErrorOperand()
+
+    return v
+
+
 ARITHMETIC_FUNCTIONS = {
     '+': add_func,
     '-': subtract_func,
@@ -618,39 +687,54 @@ ARITHMETIC_FUNCTIONS = {
 EXCEL_FUNCTIONS = {}
 EXCEL_FUNCTIONS.update(ARITHMETIC_FUNCTIONS)
 
+EXCEL_FUNCTIONS['ABS'] = abs_function
+EXCEL_FUNCTIONS['AND'] = and_function
+EXCEL_FUNCTIONS['AVERAGE'] = average_function
+EXCEL_FUNCTIONS['AVERAGEIFS'] = average_ifs_function
+
+EXCEL_FUNCTIONS['CONCATENATE'] = concatenate
+EXCEL_FUNCTIONS['COUNT'] = count_function
+EXCEL_FUNCTIONS['COUNTA'] = counta_function
+EXCEL_FUNCTIONS['COUNTIF'] = countif_function
+EXCEL_FUNCTIONS['COUNTIFS'] = count_ifs_function
+EXCEL_FUNCTIONS['COUNTBLANK'] = count_blank_function
+
+EXCEL_FUNCTIONS['FLOOR'] = floor_function
+
+EXCEL_FUNCTIONS['IF'] = if_func
+EXCEL_FUNCTIONS['IFERROR'] = if_error_func
+EXCEL_FUNCTIONS['INDEX'] = index_function
+EXCEL_FUNCTIONS['ISBLANK'] = is_blank_func
+EXCEL_FUNCTIONS['ISERROR'] = is_error_func
+
+EXCEL_FUNCTIONS['LARGE'] = large_function
+EXCEL_FUNCTIONS['LEN'] = len_func
+EXCEL_FUNCTIONS['LEFT'] = left_func
+
+EXCEL_FUNCTIONS['MATCH'] = match_function
+EXCEL_FUNCTIONS['MAX'] = max_func
+EXCEL_FUNCTIONS['MID'] = mid_func
+EXCEL_FUNCTIONS['MIN'] = min_func
+EXCEL_FUNCTIONS['MOD'] = mod_func
+
+EXCEL_FUNCTIONS['NOT'] = not_func
+
+EXCEL_FUNCTIONS['OFFSET'] = offset_function
+EXCEL_FUNCTIONS['OR'] = or_function
+
+EXCEL_FUNCTIONS['RIGHT'] = right_func
+EXCEL_FUNCTIONS['ROUND'] = round_function
+EXCEL_FUNCTIONS['ROUNDDOWN'] = round_down_function
+
+EXCEL_FUNCTIONS['SEARCH'] = search_func
+EXCEL_FUNCTIONS['SMALL'] = small_function
+EXCEL_FUNCTIONS['SUBSTITUTE'] = substitute_func
 EXCEL_FUNCTIONS['SUM'] = sum_func
 EXCEL_FUNCTIONS['SUMIF'] = sum_if_function
 EXCEL_FUNCTIONS['SUMIFS'] = sum_ifs_function
-EXCEL_FUNCTIONS['MOD'] = mod_func
-EXCEL_FUNCTIONS['IF'] = if_func
-EXCEL_FUNCTIONS['IFERROR'] = if_error_func
-EXCEL_FUNCTIONS['ISERROR'] = is_error_func
-EXCEL_FUNCTIONS['MAX'] = max_func
-EXCEL_FUNCTIONS['MIN'] = min_func
-EXCEL_FUNCTIONS['LEFT'] = left_func
-EXCEL_FUNCTIONS['RIGHT'] = right_func
-EXCEL_FUNCTIONS['MID'] = mid_func
-EXCEL_FUNCTIONS['ISBLANK'] = is_blank_func
-EXCEL_FUNCTIONS['OR'] = or_function
-EXCEL_FUNCTIONS['AND'] = and_function
-EXCEL_FUNCTIONS['NOT'] = not_func
-EXCEL_FUNCTIONS['ROUND'] = round_function
-EXCEL_FUNCTIONS['ROUNDDOWN'] = round_down_function
-EXCEL_FUNCTIONS['FLOOR'] = floor_function
-EXCEL_FUNCTIONS['COUNT'] = count_function
-EXCEL_FUNCTIONS['COUNTIF'] = countif_function
-EXCEL_FUNCTIONS['COUNTBLANK'] = count_blank_function
-EXCEL_FUNCTIONS['COUNTA'] = counta_function
-EXCEL_FUNCTIONS['ABS'] = abs_function
-EXCEL_FUNCTIONS['OFFSET'] = offset_function
-EXCEL_FUNCTIONS['MATCH'] = match_function
-EXCEL_FUNCTIONS['AVERAGE'] = average_function
-EXCEL_FUNCTIONS['AVERAGEIFS'] = average_ifs_function
+
+EXCEL_FUNCTIONS['TRIM'] = trim_func
+
 EXCEL_FUNCTIONS['VLOOKUP'] = vlookup_function
-EXCEL_FUNCTIONS['SMALL'] = small_function
-EXCEL_FUNCTIONS['LARGE'] = large_function
-EXCEL_FUNCTIONS['COUNTIFS'] = count_ifs_function
-EXCEL_FUNCTIONS['CONCATENATE'] = concatenate
-EXCEL_FUNCTIONS['INDEX'] = index_function
-EXCEL_FUNCTIONS['SUBSTITUTE'] = substitute_func
-EXCEL_FUNCTIONS['SEARCH'] = search_func
+
+EXCEL_FUNCTIONS['YEARFRAC'] = year_frac
